# friendliai

Developer-friendly & type-safe Python SDK specifically catered to leverage *friendliai* API.

<div align="left">
    <a href="https://www.speakeasy.com/?utm_source=friendliai&utm_campaign=python"><img src="https://custom-icon-badges.demolab.com/badge/-Built%20By%20Speakeasy-212015?style=for-the-badge&logoColor=FBE331&logo=speakeasy&labelColor=545454" /></a>
    <a href="https://opensource.org/licenses/MIT">
        <img src="https://img.shields.io/badge/License-MIT-blue.svg" style="width: 100px; height: 28px;" />
    </a>
</div>


## 🏗 **Welcome to your new SDK!** 🏗

It has been generated successfully based on your OpenAPI spec. However, it is not yet ready for production use. Here are some next steps:
- [ ] 🛠 Make your SDK feel handcrafted by [customizing it](https://www.speakeasy.com/docs/customize-sdks)
- [ ] ♻️ Refine your SDK quickly by iterating locally with the [Speakeasy CLI](https://github.com/speakeasy-api/speakeasy)
- [ ] 🎁 Publish your SDK to package managers by [configuring automatic publishing](https://www.speakeasy.com/docs/advanced-setup/publish-sdks)
- [ ] ✨ When ready to productionize, delete this section from the README

<!-- Start Summary [summary] -->
## Summary

Friendli Endpoints API Reference: This is an OpenAPI reference of Friendli Endpoints API.
<!-- End Summary [summary] -->

<!-- Start Table of Contents [toc] -->
## Table of Contents

* [SDK Installation](https://github.com/friendliai/friendli-python-test/blob/master/#sdk-installation)
* [IDE Support](https://github.com/friendliai/friendli-python-test/blob/master/#ide-support)
* [SDK Example Usage](https://github.com/friendliai/friendli-python-test/blob/master/#sdk-example-usage)
* [Available Resources and Operations](https://github.com/friendliai/friendli-python-test/blob/master/#available-resources-and-operations)
* [Server-sent event streaming](https://github.com/friendliai/friendli-python-test/blob/master/#server-sent-event-streaming)
* [Retries](https://github.com/friendliai/friendli-python-test/blob/master/#retries)
* [Error Handling](https://github.com/friendliai/friendli-python-test/blob/master/#error-handling)
* [Server Selection](https://github.com/friendliai/friendli-python-test/blob/master/#server-selection)
* [Custom HTTP Client](https://github.com/friendliai/friendli-python-test/blob/master/#custom-http-client)
* [Authentication](https://github.com/friendliai/friendli-python-test/blob/master/#authentication)
* [Debugging](https://github.com/friendliai/friendli-python-test/blob/master/#debugging)
<!-- End Table of Contents [toc] -->

<!-- Start SDK Installation [installation] -->
## SDK Installation

The SDK can be installed with either *pip* or *poetry* package managers.

### PIP

*PIP* is the default package installer for Python, enabling easy installation and management of packages from PyPI via the command line.

```bash
pip install friendliai
```

### Poetry

*Poetry* is a modern tool that simplifies dependency management and package publishing by using a single `pyproject.toml` file to handle project metadata and dependencies.

```bash
poetry add friendliai
```
<!-- End SDK Installation [installation] -->

<!-- Start IDE Support [idesupport] -->
## IDE Support

### PyCharm

Generally, the SDK will work well with most IDEs out of the box. However, when using PyCharm, you can enjoy much better integration with Pydantic by installing an additional plugin.

- [PyCharm Pydantic Plugin](https://docs.pydantic.dev/latest/integrations/pycharm/)
<!-- End IDE Support [idesupport] -->

<!-- Start SDK Example Usage [usage] -->
## SDK Example Usage

### Example

```python
# Synchronous Example
import friendliai
from friendliai import Friendli
import os

s = Friendli(
    bearer_auth=os.getenv("FRIENDLI_BEARER_AUTH", ""),
)

res = s.inference.chat_completion(chat_completion_request_body={
    "model": "meta-llama-3.1-8b-instruct",
    "messages": [
        {
            "role": friendliai.Role.SYSTEM,
            "content": "You are a helpful assistant.",
        },
        {
            "role": friendliai.UserMessageRole.USER,
            "content": "Hello!",
        },
    ],
    "max_tokens": 200,
})

if res is not None:
    for event in res:
        # handle event
        print(event, flush=True)
```

</br>

The same SDK client can also be used to make asychronous requests by importing asyncio.
```python
# Asynchronous Example
import asyncio
import friendliai
from friendliai import Friendli
import os

async def main():
    s = Friendli(
        bearer_auth=os.getenv("FRIENDLI_BEARER_AUTH", ""),
    )
    res = await s.inference.chat_completion_async(chat_completion_request_body={
        "model": "meta-llama-3.1-8b-instruct",
        "messages": [
            {
                "role": friendliai.Role.SYSTEM,
                "content": "You are a helpful assistant.",
            },
            {
                "role": friendliai.UserMessageRole.USER,
                "content": "Hello!",
            },
        ],
        "max_tokens": 200,
    })
    if res is not None:
        for event in res:
            # handle event
            print(event, flush=True)

asyncio.run(main())
```
<!-- End SDK Example Usage [usage] -->

<!-- Start Available Resources and Operations [operations] -->
## Available Resources and Operations

<details open>
<summary>Available methods</summary>


### [inference](https://github.com/friendliai/friendli-python-test/blob/master/docs/sdks/inference/README.md)

* [chat_completion](https://github.com/friendliai/friendli-python-test/blob/master/docs/sdks/inference/README.md#chat_completion) - Chat completion
* [completion](https://github.com/friendliai/friendli-python-test/blob/master/docs/sdks/inference/README.md#completion) - Completion
* [tokenization](https://github.com/friendliai/friendli-python-test/blob/master/docs/sdks/inference/README.md#tokenization) - Tokenization
* [detokenization](https://github.com/friendliai/friendli-python-test/blob/master/docs/sdks/inference/README.md#detokenization) - Detokenization

### [serverless](https://github.com/friendliai/friendli-python-test/blob/master/docs/sdks/serverless/README.md)

* [tool_assisted_chat_completion](https://github.com/friendliai/friendli-python-test/blob/master/docs/sdks/serverless/README.md#tool_assisted_chat_completion) - Tool assisted chat completion

</details>
<!-- End Available Resources and Operations [operations] -->

<!-- Start Server-sent event streaming [eventstream] -->
## Server-sent event streaming

[Server-sent events][mdn-sse] are used to stream content from certain
operations. These operations will expose the stream as [Generator][generator] that
can be consumed using a simple `for` loop. The loop will
terminate when the server no longer has any events to send and closes the
underlying connection.

```python
import friendliai
from friendliai import Friendli
import os

s = Friendli(
    bearer_auth=os.getenv("FRIENDLI_BEARER_AUTH", ""),
)

res = s.inference.chat_completion(chat_completion_request_body={
    "model": "meta-llama-3.1-8b-instruct",
    "messages": [
        {
            "role": friendliai.Role.SYSTEM,
            "content": "You are a helpful assistant.",
        },
        {
            "role": friendliai.UserMessageRole.USER,
            "content": "Hello!",
        },
    ],
    "max_tokens": 200,
})

if res is not None:
    for event in res:
        # handle event
        print(event, flush=True)

```

[mdn-sse]: https://developer.mozilla.org/en-US/docs/Web/API/Server-sent_events/Using_server-sent_events
[generator]: https://wiki.python.org/moin/Generators
<!-- End Server-sent event streaming [eventstream] -->

<!-- Start Retries [retries] -->
## Retries

Some of the endpoints in this SDK support retries. If you use the SDK without any configuration, it will fall back to the default retry strategy provided by the API. However, the default retry strategy can be overridden on a per-operation basis, or across the entire SDK.

To change the default retry strategy for a single API call, simply provide a `RetryConfig` object to the call:
```python
from friendli.utils import BackoffStrategy, RetryConfig
import friendliai
from friendliai import Friendli
import os

s = Friendli(
    bearer_auth=os.getenv("FRIENDLI_BEARER_AUTH", ""),
)

res = s.inference.chat_completion(chat_completion_request_body={
    "model": "meta-llama-3.1-8b-instruct",
    "messages": [
        {
            "role": friendliai.Role.SYSTEM,
            "content": "You are a helpful assistant.",
        },
        {
            "role": friendliai.UserMessageRole.USER,
            "content": "Hello!",
        },
    ],
    "max_tokens": 200,
},
    RetryConfig("backoff", BackoffStrategy(1, 50, 1.1, 100), False))

if res is not None:
    for event in res:
        # handle event
        print(event, flush=True)

```

If you'd like to override the default retry strategy for all operations that support retries, you can use the `retry_config` optional parameter when initializing the SDK:
```python
from friendli.utils import BackoffStrategy, RetryConfig
import friendliai
from friendliai import Friendli
import os

s = Friendli(
    retry_config=RetryConfig("backoff", BackoffStrategy(1, 50, 1.1, 100), False),
    bearer_auth=os.getenv("FRIENDLI_BEARER_AUTH", ""),
)

res = s.inference.chat_completion(chat_completion_request_body={
    "model": "meta-llama-3.1-8b-instruct",
    "messages": [
        {
            "role": friendliai.Role.SYSTEM,
            "content": "You are a helpful assistant.",
        },
        {
            "role": friendliai.UserMessageRole.USER,
            "content": "Hello!",
        },
    ],
    "max_tokens": 200,
})

if res is not None:
    for event in res:
        # handle event
        print(event, flush=True)

```
<!-- End Retries [retries] -->

<!-- Start Error Handling [errors] -->
## Error Handling

Handling errors in this SDK should largely match your expectations. All operations return a response object or raise an exception.

By default, an API error will raise a models.SDKError exception, which has the following properties:

| Property        | Type             | Description           |
|-----------------|------------------|-----------------------|
| `.status_code`  | *int*            | The HTTP status code  |
| `.message`      | *str*            | The error message     |
| `.raw_response` | *httpx.Response* | The raw HTTP response |
| `.body`         | *str*            | The response content  |

When custom error responses are specified for an operation, the SDK may also raise their associated exceptions. You can refer to respective *Errors* tables in SDK docs for more details on possible exception types for each operation. For example, the `chat_completion_async` method may raise the following exceptions:

| Error Type      | Status Code     | Content Type    |
| --------------- | --------------- | --------------- |
| models.SDKError | 4XX, 5XX        | \*/\*           |

### Example

```python
import friendliai
from friendliai import Friendli, models
import os

s = Friendli(
    bearer_auth=os.getenv("FRIENDLI_BEARER_AUTH", ""),
)

res = None
try:
    res = s.inference.chat_completion(chat_completion_request_body={
        "model": "meta-llama-3.1-8b-instruct",
        "messages": [
            {
                "role": friendliai.Role.SYSTEM,
                "content": "You are a helpful assistant.",
            },
            {
                "role": friendliai.UserMessageRole.USER,
                "content": "Hello!",
            },
        ],
        "max_tokens": 200,
    })

    if res is not None:
        for event in res:
            # handle event
            print(event, flush=True)

except models.SDKError as e:
    # handle exception
    raise(e)
```
<!-- End Error Handling [errors] -->

<!-- Start Server Selection [server] -->
## Server Selection

### Select Server by Index

You can override the default server globally by passing a server index to the `server_idx: int` optional parameter when initializing the SDK client instance. The selected server will then be used as the default on the operations that use it. This table lists the indexes associated with the available servers:

| # | Server | Variables |
| - | ------ | --------- |
| 0 | `https://inference.friendli.ai` | None |
| 1 | `https://inference.friendli.ai/dedicated` | None |

#### Example

```python
import friendliai
from friendliai import Friendli
import os

s = Friendli(
    server_idx=1,
    bearer_auth=os.getenv("FRIENDLI_BEARER_AUTH", ""),
)

res = s.inference.chat_completion(chat_completion_request_body={
    "model": "meta-llama-3.1-8b-instruct",
    "messages": [
        {
            "role": friendliai.Role.SYSTEM,
            "content": "You are a helpful assistant.",
        },
        {
            "role": friendliai.UserMessageRole.USER,
            "content": "Hello!",
        },
    ],
    "max_tokens": 200,
})

if res is not None:
    for event in res:
        # handle event
        print(event, flush=True)

```


### Override Server URL Per-Client

The default server can also be overridden globally by passing a URL to the `server_url: str` optional parameter when initializing the SDK client instance. For example:
```python
import friendliai
from friendliai import Friendli
import os

s = Friendli(
    server_url="https://inference.friendli.ai",
    bearer_auth=os.getenv("FRIENDLI_BEARER_AUTH", ""),
)

res = s.inference.chat_completion(chat_completion_request_body={
    "model": "meta-llama-3.1-8b-instruct",
    "messages": [
        {
            "role": friendliai.Role.SYSTEM,
            "content": "You are a helpful assistant.",
        },
        {
            "role": friendliai.UserMessageRole.USER,
            "content": "Hello!",
        },
    ],
    "max_tokens": 200,
})

if res is not None:
    for event in res:
        # handle event
        print(event, flush=True)

```

### Override Server URL Per-Operation

The server URL can also be overridden on a per-operation basis, provided a server list was specified for the operation. For example:
```python
import friendliai
from friendliai import Friendli
import os

s = Friendli(
    bearer_auth=os.getenv("FRIENDLI_BEARER_AUTH", ""),
)

res = s.serverless.tool_assisted_chat_completion(tool_assisted_completion_request_body={
    "model": "meta-llama-3.1-8b-instruct",
    "messages": [
        {
            "role": friendliai.Role.SYSTEM,
            "content": "You are a helpful assistant.",
        },
        {
            "role": friendliai.UserMessageRole.USER,
            "content": "Hello!",
        },
    ],
    "max_tokens": 200,
    "tools": [
        {
            "type": friendliai.OtherBuiltInToolType.MATH_CALCULATOR,
        },
        {
            "type": friendliai.OtherBuiltInToolType.WEB_URL,
        },
    ],
}, server_url="https://inference.friendli.ai")

if res is not None:
    for event in res:
        # handle event
        print(event, flush=True)

```
<!-- End Server Selection [server] -->

<!-- Start Custom HTTP Client [http-client] -->
## Custom HTTP Client

The Python SDK makes API calls using the [httpx](https://www.python-httpx.org/) HTTP library.  In order to provide a convenient way to configure timeouts, cookies, proxies, custom headers, and other low-level configuration, you can initialize the SDK client with your own HTTP client instance.
Depending on whether you are using the sync or async version of the SDK, you can pass an instance of `HttpClient` or `AsyncHttpClient` respectively, which are Protocol's ensuring that the client has the necessary methods to make API calls.
This allows you to wrap the client with your own custom logic, such as adding custom headers, logging, or error handling, or you can just pass an instance of `httpx.Client` or `httpx.AsyncClient` directly.

For example, you could specify a header for every request that this sdk makes as follows:
```python
from friendliai import Friendli
import httpx

http_client = httpx.Client(headers={"x-custom-header": "someValue"})
s = Friendli(client=http_client)
```

or you could wrap the client with your own custom logic:
```python
from friendliai import Friendli
from friendliai.httpclient import AsyncHttpClient
import httpx

class CustomClient(AsyncHttpClient):
    client: AsyncHttpClient

    def __init__(self, client: AsyncHttpClient):
        self.client = client

    async def send(
        self,
        request: httpx.Request,
        *,
        stream: bool = False,
        auth: Union[
            httpx._types.AuthTypes, httpx._client.UseClientDefault, None
        ] = httpx.USE_CLIENT_DEFAULT,
        follow_redirects: Union[
            bool, httpx._client.UseClientDefault
        ] = httpx.USE_CLIENT_DEFAULT,
    ) -> httpx.Response:
        request.headers["Client-Level-Header"] = "added by client"

        return await self.client.send(
            request, stream=stream, auth=auth, follow_redirects=follow_redirects
        )

    def build_request(
        self,
        method: str,
        url: httpx._types.URLTypes,
        *,
        content: Optional[httpx._types.RequestContent] = None,
        data: Optional[httpx._types.RequestData] = None,
        files: Optional[httpx._types.RequestFiles] = None,
        json: Optional[Any] = None,
        params: Optional[httpx._types.QueryParamTypes] = None,
        headers: Optional[httpx._types.HeaderTypes] = None,
        cookies: Optional[httpx._types.CookieTypes] = None,
        timeout: Union[
            httpx._types.TimeoutTypes, httpx._client.UseClientDefault
        ] = httpx.USE_CLIENT_DEFAULT,
        extensions: Optional[httpx._types.RequestExtensions] = None,
    ) -> httpx.Request:
        return self.client.build_request(
            method,
            url,
            content=content,
            data=data,
            files=files,
            json=json,
            params=params,
            headers=headers,
            cookies=cookies,
            timeout=timeout,
            extensions=extensions,
        )

s = Friendli(async_client=CustomClient(httpx.AsyncClient()))
```
<!-- End Custom HTTP Client [http-client] -->

<!-- Start Authentication [security] -->
## Authentication

### Per-Client Security Schemes

This SDK supports the following security scheme globally:

| Name                   | Type                   | Scheme                 | Environment Variable   |
| ---------------------- | ---------------------- | ---------------------- | ---------------------- |
| `bearer_auth`          | http                   | HTTP Bearer            | `FRIENDLI_BEARER_AUTH` |

To authenticate with the API the `bearer_auth` parameter must be set when initializing the SDK client instance. For example:
```python
import friendliai
from friendliai import Friendli
import os

s = Friendli(
    bearer_auth=os.getenv("FRIENDLI_BEARER_AUTH", ""),
)

res = s.inference.chat_completion(chat_completion_request_body={
    "model": "meta-llama-3.1-8b-instruct",
    "messages": [
        {
            "role": friendliai.Role.SYSTEM,
            "content": "You are a helpful assistant.",
        },
        {
            "role": friendliai.UserMessageRole.USER,
            "content": "Hello!",
        },
    ],
    "max_tokens": 200,
})

if res is not None:
    for event in res:
        # handle event
        print(event, flush=True)

```
<!-- End Authentication [security] -->

<!-- Start Debugging [debug] -->
## Debugging

You can setup your SDK to emit debug logs for SDK requests and responses.

You can pass your own logger class directly into your SDK.
```python
from friendliai import Friendli
import logging

logging.basicConfig(level=logging.DEBUG)
s = Friendli(debug_logger=logging.getLogger("friendliai"))
```

You can also enable a default debug logger by setting an environment variable `FRIENDLI_DEBUG` to true.
<!-- End Debugging [debug] -->

<!-- Placeholder for Future Speakeasy SDK Sections -->

# Development

## Maturity

This SDK is in beta, and there may be breaking changes between versions without a major version update. Therefore, we recommend pinning usage
to a specific package version. This way, you can install the same version each time without breaking changes unless you are intentionally
looking for the latest version.

## Contributions

While we value open-source contributions to this SDK, this library is generated programmatically. Any manual changes added to internal files will be overwritten on the next generation. 
We look forward to hearing your feedback. Feel free to open a PR or an issue with a proof of concept and we'll do our best to include it in a future release. 

### SDK Created by [Speakeasy](https://www.speakeasy.com/?utm_source=friendliai&utm_campaign=python)
